% Function: [W_reg,R,L,t,scale_W] = register_2D3D_exact_deformable(W,S,iter)
%
% The function register a deformable model with k basis shapes to a set of
% matched 2D image coordinates. 
%
% For more details and references read our paper:
%
% @conference{DelBue:etal:2009, 
%  title={2D-3D Registration Of Deformable Shapes With Manifold Projection}, 
%  author={A. {Del Bue} and M. Stosic and M. Dodig and J. Xavier}, 
%  booktitle = "International Conference on Image Processing (ICIP 2009), Cairo, Egypt", 
%  year={2009}}
%  
%  Please reference this article if you use this code for your projects
% Input
%
% W: a 2 x p matrix containing the 2D image coordinates 
% S: a 3k x p matrix containing the 3D model with k basis shapes
% iter: number of iterations (normally set to 1)
%
% Output
%
% W_reg: a 2 x p matrix containing the registered coordinates given S
% R: a 2 x 3 matrix containing the estimated ortographic camera matrix
% L: a k-vector containing the configuration weights
% t: the 2-vector with the image translation
% scale_W: a scalar which scale the data for normalization purposes

function [W_reg,R,L,t,scale_W] = register_2D3D_exact_deformable(W,S,iter)

% Copyright (C) 2009 Alessio Del Bue, Marko Stoi?, Marija Dodig, Joo
% Xavier.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


if isempty(iter)
    iter = 1;
end

p = size(W,2);
r = size(S,1);

% normalization
scale_W=max(max(abs(W)));
W = W/scale_W;

scale_S=max(max(abs(S)));
S = S/scale_S;

t = zeros(2,1);
M = zeros(2,r);

S_homog = [S;ones(1,p)];
clear S

% initial affine estimate 
M_tr = W * S_homog' * inv(S_homog*S_homog');
t=M_tr(1:2,r+1);

% Iterations are only needed if the 3D deformable model S is only an
% approximated description of the image data. In general, a single iteration
% is sufficient.

for i =1:iter
    
    % affine estimate 
    W_reg = W - t*ones(1,p);
    M = W_reg * S_homog(1:r,:)' * inv(S_homog(1:r,:)*S_homog(1:r,:)');
    % projection
    [R,L]=register_2D3D_exact_project(M_tr(1:2,1:r));
    P = fact_plugLR(L',R);
    t = mean((W - P * S_homog(1:r,:))')';
    W_iter = fact_reproject(L',R,S_homog(1:r,:),t,1);
    
%     plot(W(1,:), W(2,:),'r.');
%     hold on
%     plot(W_iter(1,:), W_iter(2,:),'kx');
%     hold off
%     drawnow
%     pause

end
W_reg = scale_W*W_iter;