% Function: [Ri,li]=register_2D3D_exact_project(M)
%
% Motion manifold projection given an affine estimate M. This function
% requires SeDuMi correctly installed in your machine
% (http://sedumi.ie.lehigh.edu/).
%
% For more details and references read our paper:
%
% @conference{DelBue:etal:2009, 
%  title={2D-3D Registration Of Deformable Shapes With Manifold Projection}, 
%  author={A. {Del Bue} and M. Stosic and M. Dodig and J. Xavier}, 
%  booktitle = "International Conference on Image Processing (ICIP 2009), Cairo, Egypt", 
%  year={2009}}
%  
%  Please reference this article if you use this code for your projects
% 
% Input
%
% M: 2x3k affine motion matrix 
%
% Output
%
% Ri: 2x3 estimated orthographic camera matrix.
% li: k-vector containing the configuration weights for the deformable
% model

function [Ri,li]=register_2D3D_exact_project(M)

% Copyright (C) 2009 Alessio Del Bue, Marko Stoi?, Marija Dodig, Joo
% Xavier.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


% Search for the closest matrix to M that is in the form
% [l1 Ri | l2 Ri | ... | lk Ri]
% where Ri is Stiefel and l1..lk are the deformable shape coefficients
A=zeros(6,6);
K=size(M,2)/3;

for i=1:K
    m=vec(M(:,3*i-2:3*i)'); %CONTROL transpose
    A=A - m*m';
end

X = sdpvar(6,6);

F = set(X >= 0) + set(trace(X(1:3,1:3)) == 1) + set(trace(X(4:6,4:6)) == 1) + set(trace(X(1:3,4:6)) == 0);


B = X(1:3,4:6); 
D = B-B';
v = [ D(2,3) ; D(3,1) ; D(1,2) ];
F = F + set([ eye(3)-X(1:3,1:3)-X(4:6,4:6) v ; v' 1 ] >= 0);

% solvesdp(F,trace(A*X),sdpsettings('verbose',0, 'cachesolvers',1));
solvesdp(F,trace(A*X),sdpsettings('verbose',1, 'cachesolvers',0));
Xs = double(X);

[U,D]=eig(Xs);
[sd,ix]=sort(diag(D));
u=U(:,ix(end));
Ri=sqrt(sd(end))*[u(1:3,1) u(4:6,1) ];
Ri=Ri';
li=zeros(1,K);
li(1) =  (1/2) * trace(M(:,1:3)'*Ri);

if li(1) < 0 
    Ri=-Ri;
    li(1)=-li(1);
end

for j=2:K
    li(j)= (1/2) * trace(M(:,3*j-2:3*j)'*Ri);

end


end
